#!/bin/bash -eux
set -o pipefail
# Licensed to Elasticsearch B.V. under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Elasticsearch B.V. licenses this file to you under
# the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# 	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
# Author: Derek Ditch <dcode@users.noreply.github.com>
# Date:   2022-11-16

# This script depends on the following:
# - bash
# - GNU awk
# - base64 from GNU Coreutils (the macOS version likely won't work)
# - curl
# - rev
# - sha256sum from GNU Coreutils
# - vt cli from https://github.com/VirusTotal/vt-cli
# - ikdasm from the Mono project

# You can make this anything you want, just trying to blend in with the winds of the Internet
USER_AGENT="Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)"

# Create url1.txt and url2.txt with unique contents of respective URLs, skipping the header line
awk 'NR!=1 { print $3}' <output.tsv | sort -u >url1.txt
awk 'NR!=1 { print $3}' <output.tsv | sort -u >url2.txt

# Create some dirs to organize our analysis
mkdir -p url1 url2 dll

cd url1 || exit 1

echo "Attempting to download files from URL1. This may take some time."
ctr=1
while read -r line; do
    echo "curl -v -A \"${USER_AGENT}\" -o \"file-${ctr}\" -L --connect-timeout 10 \"${line}\" 2>>\"log-${ctr}.txt\"" >"log-${ctr}.txt"
    curl -v -A "${USER_AGENT}" -o "file-${ctr}" -L --connect-timeout 10 "${line}" 2>>"log-${ctr}.txt"
    ctr=$((ctr + 1))
done <../url1.txt

# For url1, decode the text as-is
for entry in $(file file-?? | awk -F": " '$2 ~ /^ASCII text.*very long lines/  { print $1 }'); do
    base64 -d <"${entry}" >"../dll/url1-${entry}.dll"
done

cd ../url2 || exit 2
echo "Attempting to download files from URL2. This may take some time."
ctr=1
while read -r line; do
    echo "curl -v -A \"GoogleBot/1.2\" -o \"file-${ctr}\" -L --connect-timeout 10 \"${line}\" 2>>\"log-${ctr}.txt\"" >"log-${ctr}.txt"
    curl -v -A "GoogleBot/1.2" -o "file-${ctr}" -L --connect-timeout 10 "${line}" 2>>"log-${ctr}.txt"
    ctr=$((ctr + 1))
done <../url2.txt

# For url2, reverse the text before decoding
for entry in $(file file* | awk -F": " '$2 ~ /^ASCII text.*very long lines/  { print $1 }'); do
    rev <"${entry}" | base64 -d >"../dll/url2-${entry}.dll"
done

cd ../dll || exit 3

# Exit here if we didn't find any DLLs
if [ ! "$(find . -name '*.dll' | wc -l)" -gt 0 ]; then
    echo "No DLLs were captured"
    exit
fi

echo "Attempting to search VirusTotal for captured DLLs"
# Search VT
# Truncate the result files before we begin
echo -n >vt-found.txt
echo -n >vt-notfound.txt
for entry in *.dll; do
    hash=$(sha256sum "${entry}" | awk '{print $1}')
    echo -n "Searching for ${hash} in VT:  "
    vt search "${hash}" >"${entry}.vt.yml"

    if [ -s "${entry}.vt.yml" ]; then
        echo "Found"
        echo -e "${hash}\t${entry}" >>vt-found.txt
    else
        echo "Not Found"
        rm "${entry}.vt.yml"
        echo -e "${hash}\t${entry}" >>vt-notfound.txt
    fi
done

# Truncate the file before we append results below
echo -n >possible-infra.txt
# For .Net assemblies, search for URLs and/or domains, could definitely expand this grep pattern
for item in $(file file*.dll | awk -F": " '$2 ~ /Mono\/.Net assembly/  { print $1 }'); do
    echo "${item} ========================" | tee -a possible-infra.txt
    ikdasm "${item}" | grep -E '(\.(org|com|net|ly))|((yl|ten|moc|gro)\.)|("http|ptth")' | tee -a possible-infra.txt
done

# Do a regular strings dump on PE32 files that are not .Net assemblies
for item in $(file file*.dll | awk -F": " '$2 ~ /PE32 executable/ && $2 !~ /Mono\/.Net assembly/  { print $1 }'); do
    echo "${item} ========================" | tee -a possible-infra.txt
    strings -a "${item}" | grep -E '(\.(org|com|net|ly))|((yl|ten|moc|gro)\.)|("http|ptth")' | tee -a possible-infra.txt
done
